package no.inspera.services.pedal;

import no.inspera.services.pedal.utils.StringUtils;

/**
 *
 * A <tt>LearningObject</tt> is an entity used for learning, education or training.
 *
 * @author <a href="mailto:naimdjon@gmail.com">Naimdjon Takhirov</a>
 * @version $Id: LearningObject.java 45074 2008-05-29 10:04:54Z naimdjon $
 * @since 20.12.2007
 */
public class LearningObject {
    /**
     * Identifier of this LearningObject
     */
    private long id;

    /**
     * The level metadata element. LearningObject is tagged with different level (e.g. easy, intermediate, advanced.)
     */
    private String level;

    /**
     * The learningStyle  metadata element. LearningObject is tagged with any one of VAK entry (visual, auditory, kinesthetic.)
     */
    private String learningStyle;

    /**
     * The string defining the type of this LearningObject. The type can be example, image, video, sound or assessment.
     */
    private String type;

    /**
     * Boolean indicator whether or not this LearningObject is exercise.
     */
    private boolean isExercise;

    /**
     * Constructs a new LearningObject with specified id and type.
     *
     * @param id the identifier of this LearningObject.
     * @param type the content type of this LearningObject
     */
    public LearningObject(long id, String type) {
        this.id = id;
        this.type = type;
    }

    /**
     * Setter for id property
     * @param id new id property.
     */
    public void setId(long id) {
        this.id = id;
    }

    /**
     * Setter for level property
     * @param level new level property
     */
    public void setLevel(String level) {
        this.level = level;
    }

    /**
     * Setter for learningStyle property
     * @param learningStyle new learningStyle property
     */
    public void setLearningStyle(String learningStyle) {
        this.learningStyle = learningStyle;
    }

    /**
     * Setter for type property
     * @param type new type property
     */
    public void setType(String type) {
        this.type = type;
    }

    /**
     * Getter for Id
     * 
     * @return Id of this LearningObject
     */
    public long getId() {
        return id;
    }

    /**
     * Getter for level property
     * @return level of this LearningObject
     */
    public String getLevel() {
        return level;
    }

    /**
     * Getter for learningStyle property
     * @return learningStyle of this LearningObject
     */
    public String getLearningStyle() {
        return learningStyle;
    }

    /**
     * Checks whether this LearningObject is example.
     * @return  true if this LearningObject has examlpe as type, false otherwise
     */
    public boolean isExample() {
        return StringUtils.equals(this.type, "content_document_example");
    }

    /**
     * Checks whether this LearningObject is theory.
     * @return  true if this LearningObject has theory as type, false otherwise
     */
    public boolean isTheory() {
        return StringUtils.equals(this.type, "content_document_theory");
    }

    /**
     * Checks whether this LearningObject is exercise.
     * @return  true if this LearningObject has exercise as type, false otherwise
     */    
    public boolean isExercise() {
        return StringUtils.anyEquals(this.type, "content_activity_exercise", "content_exercise") || isExercise;
    }

    /**
     * Setter for isExercise property
     *
     * @param isExercise new value for isExercise property
     */
    public void setIsExercise(Boolean isExercise) {
        this.isExercise = isExercise;
    }

    /**
     * Returns string representation of this LearningObject
     * @return string representation of this LearningObject
     */
    public String toString() {
        return id + "-" + type;
    }

    /**
     * Indicates whether some other LearningObject is "equal to" this one.
     * <p>
     * More specifically, if two LearningObjects have the same <tt>Id</tt> identifier they are considered equal.
     * @param obj the other LearningObject
     * @return true if <tt>obj</tt> is equal to this object.
     */
    @Override
    public boolean equals(Object obj) {
        return obj != null && this.id == ((LearningObject) obj).id;
    }

    /**
     * Returns a hash code value for the LearningObject. This is among other things used to check the current state of the module.
     *
     * @return
     * @see no.inspera.services.pedal.engine.SequencingState
     */
    public int hashCode() {
        return Long.valueOf(id).hashCode();
    }


}
