package no.inspera.services.pedal;

import no.inspera.services.log.LogServices;
import no.inspera.services.pedal.engine.SequencingState;
import no.inspera.services.pedal.utils.ICollections;
import org.apache.log4j.Logger;

import java.util.Collection;
import java.util.Map;

/**
 * Module represents a course module. Any given course consists of several logical modules.
 * <p>
 * Modules are instantiated when the user logs on to the system. The state of the module is not persisted,
 * which means every time the user logs on the modules are reinitialized. Therefore it is safe, to store the sequencing state which is specific to a user.
 *
 * @author <a href="mailto:naimdjon@gmail.com">Naimdjon Takhirov</a>
 * @version $Id: Module.java 45069 2008-05-29 08:25:25Z naimdjon $
 * @since 20.12.2007
 */
public class Module {
    /**
     * Logger utility.
     */
    private final transient Logger log = LogServices.getLogger(getClass());

    /**
     * The Id of the module. The modules are unique system-wide.
     */
    private long moduleId;

    /**
     * The example learning object of this module.
     */
    private LearningObject example;

    /**
     * The theory learning object of this module.
     */
    private LearningObject theory;

    /**
     * The exercise learning object of this module.
     */
    private LearningObject exercise;

    /**
     * A collection of unknown items.
     */
    private Collection<LearningObject> unknown= ICollections.newSet();

    /**
     * All items included in this module. This hasmap contains the mapping between the Id of a LearningObject and the Learning Object itself. 
     */
    private Map<Long, LearningObject> allItems= ICollections.newMap();

    /**
     * The current state of the module. 
     */
    private SequencingState currentState;

    /**
     * Constructs a new <tt>Module</tt> object with the specified moduleId
     * @param moduleId the id of module.
     */
    public Module(long moduleId) {
        this.moduleId = moduleId;
        this.currentState = new SequencingState(this);
    }

    /**
     * Adds this item to the list of available items within this module. 
     *
     * @param learningObject item to be added
     */
    public void addItem(LearningObject learningObject){
        if(learningObject.isTheory())
            theory= learningObject;
        else if(learningObject.isExercise())
            exercise= learningObject;
        else if(learningObject.isExample())
            example= learningObject;
        else {
            log.debug("unknown:"+ learningObject);
            unknown.add(learningObject);
        }
        allItems.put(learningObject.getId(), learningObject);
    }

    /**
     * Looks up an item with specified id.
     * 
     * @param id the id of an item (either theory, exercise or example)
     * @return an item matching specified id or null if there is no item with that id
     */
    public LearningObject getItem(long id ) {
        return allItems.get(id);
    }

    /**
     * Returns all items within this module.
     *
     * @return all items in this module.
     */
    public Collection<LearningObject> getAllItems() {
        return this.allItems.values();
    }

    /**
     * Getter for example.
     * @return example
     */
    public LearningObject getExample() {
        return example;
    }

    /**
     * Getter for theory
     * @return theory
     */
    public LearningObject getTheory() {
        return theory;
    }

    /**
     * Getter for exercise
     * @return exercise or null if exercise is not available
     */
    public LearningObject getExercise() {
        return exercise;
    }

    /**
     * Returns string representation of this object.
     * @return string representation of this object
     */
    public String toString() {
        return "ex:"+example +"\n exer:"+exercise+"\n theory:"+theory;
    }

    /**
     * Returns the current state of sequenced items.
     *
     * @return current state of sequenced items.
     */
    public SequencingState getCurrentState() {
        return currentState;
    }

    /**
     * Getter for moduleId
     * 
     * @return moduleId
     */
    public long getModuleId() {
        return moduleId;
    }
}
