package no.inspera.services.pedal;

import no.inspera.applications.publish.datamodel.ContentRevision;
import no.inspera.db.BaseObjectDataAccess;
import no.inspera.db.ContentPresentationDataAccess;
import no.inspera.services.languagenegotiator.LanguageNegotiator;
import no.inspera.services.log.LogServices;
import no.inspera.services.pedal.actions.*;
import no.inspera.utils.Parameters;
import no.inspera.services.pedal.utils.ICollections;
import org.apache.log4j.Logger;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.Map;

/**
 * Servlet for personalising courses. This servlet acts as a dispatcher to all pages in the
 * personalised course.
 * <p>
 * It holds a list of possible actions that can be executed.
 * The actions implement {@link no.inspera.services.pedal.actions.Action} interface.
 * If this servlet is unable to identify the requested action the default action is login page.
 *  An example of url for the servlet:
 * <pre>
 *    http://&lt;hostname&gt;:&lt;port&gt;/pedalng?ac=view
 * </pre>
 *
 *  Which executes the view action(see {@link no.inspera.services.pedal.actions.View} for
 * more information).
 * 
 * The View action displays the requested module.  
 *
 * @author <a href="mailto:naimdjon@gmail.com">Naimdjon Takhirov</a>
 * @version $Id: PedalServlet.java 45069 2008-05-29 08:25:25Z naimdjon $
 * @since 20.12.2007  
 */
public class PedalServlet extends HttpServlet {
    /**
     * Logging utility.
     */
    private final transient Logger log = LogServices.getLogger(getClass());

    /**
     * Action map. Holds action id (string) and actual implementation.
     */
    private static Map<String, Action> pedalActions = ICollections.newMap();

    /**
     * Data access class for using content related methods.
     */
    private final ContentPresentationDataAccess cpda = ContentPresentationDataAccess.getInstance();

    static {
        pedalActions.put("idx", new Index());
        pedalActions.put("start", new Start());
        pedalActions.put("empty", new Empty());
        pedalActions.put("view", new View());
        pedalActions.put("login", new Login());
        pedalActions.put("showindex", new ShowIndex());
        pedalActions.put("tag", new TagObject());
        pedalActions.put("rate", new Rate());
    }

    /**
     * The entry gate to serve all requests.
     * 
     * @param request the current request
     * @param response reponse object where the results will be written.
     * @throws ServletException if any error occurs.
     */
    public void service(HttpServletRequest request, HttpServletResponse response) throws ServletException {
        Parameters params = Parameters.create(request);
        final long mark = params.getMarketplaceId();
        if (mark <= 0) {
            params.setMarketplaceId(PEDAL.defaultMarketplaceId);
            params.setLanguageId(LanguageNegotiator.getHighestPriorityLanguageId(mark));
        }
        final String logicalTitle = params.getParameter("pedalCourseLogicalTitle",PEDAL.defaultLogicalTitle, true);
        log.debug("logicalname:" + logicalTitle);
        final BaseObjectDataAccess boda = BaseObjectDataAccess.getInstance();
        long contentItemId =boda.getObjectIdFromLogicalName(logicalTitle, mark);
        final long lang = params.getLanguageId();
        ContentRevision revision = cpda.getLiveContentRevisionWithEffectiveLanguage(contentItemId, lang, mark);
        log.debug("revisionrevision:"+revision);
        params.setRequestAttribute(PEDAL.course, revision);
        params.logParams();
        Action action = pedalActions.get(params.getParameter("ac"));
        if (action == null || params.getUserId()<=0) action = new Login();
        Result res = action.execute(params);
        res.processResult(request, response);
    }
    
}
