package no.inspera.services.pedal.actions;

import no.inspera.applications.publish.datamodel.ContentActivityNode;
import no.inspera.applications.publish.datamodel.ContentActivityRoot;
import no.inspera.applications.publish.datamodel.ContentItem;
import no.inspera.applications.publish.datamodel.ContentRevision;
import no.inspera.applications.publish.datamodel.assessment.*;
import no.inspera.applications.publish.util.PublishConstants;
import no.inspera.db.ContentPresentationDataAccess;
import no.inspera.db.MetadataDataAccess;
import no.inspera.services.log.LogServices;
import no.inspera.services.metadataservice.*;
import no.inspera.services.pedal.PEDAL;
import no.inspera.services.pedal.engine.*;
import no.inspera.utils.Parameters;
import no.inspera.services.pedal.utils.ICollections;
import org.apache.log4j.Logger;

import java.util.Collection;
import java.util.Map;

/**
 * A configuration action for the personalised course.
 *
 * @author <a href="mailto:naimdjon@gmail.com">Naimdjon Takhirov</a>
 * @version $Id: Configure.java 45069 2008-05-29 08:25:25Z naimdjon $
 * @since 23.12.2007
 */
public class Configure implements Action {
    /**
     * Logging utility.
     */
    private final transient Logger log = LogServices.getLogger(getClass());
    /**
     * Data access that provides database access functionality.
     */
    private final ContentPresentationDataAccess cpda = ContentPresentationDataAccess.getInstance();
    /**
     * The marketplace of the user.
     */
    private long userMarketplaceId;
    /**
     * Preferred language of the user
     */
    private long userLanguageId;

    /**
     * Constructs a new Configure object.
     * @param marketplaceId the marketplaceid of the user.
     * @param languageId the language of the user.
     */
    public Configure(long marketplaceId, long languageId){
        this.userMarketplaceId=marketplaceId;
        this.userLanguageId=languageId;
    }

    /**
     * Executes the Configure object which causes the engine to be configured. Usually this is done when the user has taken prior knowledge assessment and answered learning style questionnaire.
     *
     * @param params current request parameters.
     * @return the Result of the action.
     */
    public Result execute(Parameters params) {
        final Metadata md = MetadataService.getMetadataForObject(params.getUserId());
        final SequencingMetadataItem learningStyle = getMetadataItem(md, PEDAL.learningStyle);
        final SequencingMetadataItem level=getMetadataItem(md,PEDAL.level);
        if (learningStyle != null  /*&& level!=null*/) {
            if(setupModuleLevels(params)){
                Map<Long, PEDAL.Level>levels=params.getSessionAttribute(PEDAL.moduleLevels);
            AdaptiveEngine se = EngineBuilder.create()
                    .setLearningStyle(learningStyle)
                            .setLevelMap(levels)
                            .addSequencingChain(new LevelSequencing(level))
                    .addSequencingChain(new LearningStyleSequencing(learningStyle))
                    .addEmpty()
                    .build();
                params.setSessionAttribute(PEDAL.sequencingEngine, se);
            }
            //else log.warn("setup module levels returned false!!");
        }
        //else log.warn("did not find learning style and level for the user. Not configuring..."+(learningStyle)+", le:"+level);
        return new Empty().execute(params);
    }

    /**
     * Sets up module levels for the current user.
     *
     * @param params current request parameters.
     * @return true if the setup was finished successfully, false otherwise
     */
    private boolean setupModuleLevels(Parameters params) {
        ContentRevision revision = params.getRequestAttribute(PEDAL.course);
        Map<Long, PEDAL.Level> moduleLevels = ICollections.newMap();
        final ItemValue iv = MetadataService.getMetadataForObject(revision.getObjectId()).getItemValue(PEDAL.prior_knowledge_assessment);
        final ContentRevision assessment = cpda.getLiveContentRevisionWithEffectiveLanguage(Long.valueOf(iv.getValueText()), userLanguageId, userMarketplaceId);
        final QTIUserAssessment ua = AssessmentDataAccess.getInstance().getLastUserAssessment(assessment.getContentRevisionId(), params.getUserId());
        if(ua==null) {
             log.warn("user assessment not found!");
            return false;
        }
        final QTIUserAnswer[] useranswers = ua.getUserAnswers();
        ContentActivityRoot root = (ContentActivityRoot) revision.getContentItem();
        collectPreAssessmentScores(moduleLevels, useranswers, root);
        params.setSessionAttribute(PEDAL.moduleLevels, moduleLevels);
        return true;
    }

    /**
     * Collects prior knowledge assessment scores for specific modules.
     *
     * @param moduleLevels module to level mappings.
     * @param useranswers user's answers to questions in modules.
     * @param activityNode current module activity node.
     */
    private void collectPreAssessmentScores(Map<Long, PEDAL.Level> moduleLevels, QTIUserAnswer[] useranswers, ContentActivityNode activityNode) {
        for (Long childId : activityNode.getChildren()) {
            ContentItem ci = cpda.getContentItem(childId);
            ContentActivityNode can = (ContentActivityNode) ci;
            if (!can.isMandatory() && can.getTargetId() > 0) {
                ContentItem target = cpda.getContentItem(can.getTargetId()); //folder...
                Iterable<QTIQuestion> questions = getQuestions(target, userLanguageId, userMarketplaceId);
                int maxScore = 0, userScore = 0;
                for (QTIQuestion q : questions) {
                    maxScore = q.getMaxScore().intValue() + maxScore;
                    userScore = getUserScore(q, useranswers) + userScore;
                }
                if (maxScore > 0 && userScore > 0) {
                    final long normalizedScore = (long) ((Double.valueOf(userScore) / Double.valueOf(maxScore)) * 100);
                    log.debug("maxScore:" + maxScore + ", userScore: " + userScore + ", for: " + target.getName() + ", normalizedScore:" + normalizedScore);
                    //if (normalizedScore > PEDAL.skipThreshold)
                    moduleLevels.put(can.getObjectId(), calculateLevel(normalizedScore));
                }
            }
            collectPreAssessmentScores(moduleLevels,useranswers,can);
        }
    }

    /**
     * Calculates level based on the normalized score.
     *
     * @param normalizedScore the score that is normalized.
     * @return the level based on calculated score.
     * @see {@link no.inspera.services.pedal.PEDAL.Level}
     */
    private PEDAL.Level calculateLevel(long normalizedScore) {
        PEDAL.Level level=null;
        if (normalizedScore >= PEDAL.Level.low.lowThreshold() && normalizedScore <= PEDAL.Level.low.highThreshold())
            level = PEDAL.Level.low;
        else if (normalizedScore >= PEDAL.Level.mid.lowThreshold() && normalizedScore <= PEDAL.Level.mid.highThreshold())
            level = PEDAL.Level.mid;
        else if (normalizedScore >= PEDAL.Level.advanced.lowThreshold() && normalizedScore <= PEDAL.Level.advanced.highThreshold())
            level = PEDAL.Level.advanced;
        return level;
    }

    /**
     * Gets the score of the user for the specified question
     * @param question the question.
     * @param useranswers the answers of the user.
     * @return the raw score for the question.
     */
    private int getUserScore(QTIQuestion question, QTIUserAnswer[] useranswers) {
        int userscore = 0;
        for (QTIUserAnswer ua : useranswers) {
            if (ua.getQuestionId() == question.getQuestionId()) {
                final QTIAnswerOption ao = question.getAnswerOption(ua.getAnswerOptionId());
                if (ao.getIsCorrect())
                    userscore = userscore + 1;
            }
        }
        return userscore;
    }

    /**
     * Returns an iterable object of questions for the content item.
     * @param contentItem the folder that contains all learning objects.
     * @param userLanguageId the language of the user.
     * @param marketplaceId the marketplaceid of the user.
     * @return an iterable of questions.
     */
    private Iterable<QTIQuestion> getQuestions(ContentItem contentItem, long userLanguageId, long marketplaceId) {
        Collection<QTIQuestion> questions = ICollections.newSet();
        for (Long itemId : contentItem.getChildren()) {
            ContentItem item = cpda.getContentItem(itemId);
            if (item.getContentType().startsWith(PublishConstants.OBJECTTYPE_CONTENTQUESTION)) {
                final ContentRevision rev = cpda.getLiveContentRevisionWithEffectiveLanguage(item.getObjectId(), userLanguageId, marketplaceId);
                if (rev instanceof QTIQuestion) {
                    final QTIQuestion q = (QTIQuestion) rev;
                    questions.add(q);
                }
            }
        }
        return questions;
    }

    /**
     * Constructs a new SequencingMetadataItem for the metadata element specified by <tt>itemproperty</tt>
     * @param md the metadata object
     * @param itemproperty requested itemproperty
     * @return new SequencingMetadataItem
     */
    private SequencingMetadataItem getMetadataItem(Metadata md, String itemproperty) {
        ItemProperty ip = MetadataService.getItemProperty(itemproperty);
        final ItemValue iv = md.getItemValue(ip.getItemPropertyId());
        if (iv == null) {
            return null;
        }
        
        final ItemPropertyValue ipv = ip.getItemPropertyValue(iv.getValueNumber());
        return new SequencingMetadataItem(ip, ipv);
    }


     public static String updateCompetenceLevel(long userId, String  totalScore, String maxPossibleScore) {
        try {
           if (userId <= 0) return "";
           String level = "";
           final long normalizedScore = (long) ((Double.valueOf(totalScore) / Double.valueOf(maxPossibleScore)) * 100);
           if (normalizedScore <= PEDAL.Level.low.highThreshold())
               level = "low";
           else if (normalizedScore >= PEDAL.Level.mid.lowThreshold() && normalizedScore <= PEDAL.Level.mid.highThreshold())
               level = "mid";
           else if (normalizedScore >= PEDAL.Level.advanced.highThreshold() && normalizedScore <= PEDAL.Level.advanced.highThreshold())
               level = "advanced";
           //log.info("updateCompetenceLevel totalScore:" + totalScore + ", userId:" + userId + ", maxPossibleScore:" + maxPossibleScore+", computedlevel:"+level+", normalizedscore:"+normalizedScore);
           final ItemProperty itemproperty = MetadataService.getItemProperty("level");
           final Metadata md = MetadataService.getMetadataForObject(userId);
           ItemValue iv = md.getItemValue(itemproperty.getItemPropertyId());
           for (ItemPropertyValue ipv : itemproperty.getValuesColl()) {
               if (ipv.getLogicalName().equals(level)) {
                   if (iv == null) {
                       iv = new ItemValue();
                       iv.setItemProperty(itemproperty.getItemPropertyId());
                       iv.setObjectId(userId);
                       iv.setValueNumber(ipv.getItemPropertyValueId());
                       MetadataDataAccess.getInstance().addItemValueValueNumber(iv);
                   } else {
                       iv.setValueNumber(ipv.getItemPropertyValueId());
                       MetadataDataAccess.getInstance().updateItemValue(iv);
                   }
                   //log.debug("ipv.getLogicalName():"+ipv.getLogicalName());
                   MetadataService.removeMetadataForObjectFromCache(userId);
               }
           }
       } catch (Throwable e) {
           e.printStackTrace();
       }
        return "";
    }
}