package no.inspera.services.pedal.actions;

import no.inspera.applications.publish.datamodel.*;
import no.inspera.applications.publish.util.PublishConstants;
import no.inspera.db.ContentPresentationDataAccess;
import no.inspera.services.log.LogServices;
import no.inspera.services.metadataservice.ItemPropertyValue;
import no.inspera.services.metadataservice.ItemValue;
import no.inspera.services.metadataservice.Metadata;
import no.inspera.services.metadataservice.MetadataService;
import no.inspera.services.pedal.PEDAL;
import no.inspera.services.pedal.utils.StringUtils;
import no.inspera.utils.Parameters;
import no.inspera.services.pedal.utils.ICollections;
import org.apache.log4j.Logger;
import org.apache.lucene.analysis.standard.StandardAnalyzer;
import org.apache.lucene.document.Document;
import org.apache.lucene.document.Field;
import org.apache.lucene.index.IndexWriter;
import org.apache.lucene.store.RAMDirectory;

import java.io.IOException;
import java.util.Collection;

/**
 * An Index action causes in-memory indexing the whole course structure.
 *
 * @author <a href="mailto:naimdjon@gmail.com">Naimdjon Takhirov</a>
 * @version $Id: Index.java 45074 2008-05-29 10:04:54Z naimdjon $
 * @since 18.12.2007 
 */
public class Index implements Action{
    /**
     * Logging utility.
     */
    private final transient Logger log = LogServices.getLogger(getClass());

    /**
     * The class that provides data access functionality.
     */
    private final ContentPresentationDataAccess cpda = ContentPresentationDataAccess.getInstance();

    /**
     * Executes the action which results in indexing the course structure.
     * @param params current request parameters.
     * @return Result of the index. An empty XML string to be transformed.
     */
    public Result execute(Parameters params) {
        RAMDirectory idx = new RAMDirectory();
        try {
            IndexWriter writer =new IndexWriter(idx, new StandardAnalyzer(), true);
            try {
                ContentRevision revision=(ContentRevision) params.getRequestAttribute(PEDAL.course);
                ContentActivityRoot root=(ContentActivityRoot) revision.getContentItem();
                Collection<Indexable> col= ICollections.newSet();
                for (Long child : root.getChildren())
                    addIndexables(child,params.getLanguageId(),params.getMarketplaceId(),col);
                for (Indexable ind : col) {
                    Document doc= new Document();
                    doc.add(new Field(PEDAL.Index.parentId.toString(),""+ ind.parentId,Field.Store.YES,Field.Index.UN_TOKENIZED));
                    doc.add(new Field(PEDAL.Index.id.toString(),""+ ind.id,Field.Store.YES,Field.Index.UN_TOKENIZED));
                    doc.add(new Field(PEDAL.Index.title.toString(), ind.title,Field.Store.YES,Field.Index.TOKENIZED));
                    doc.add(new Field(PEDAL.Index.type.toString(), ind.type,Field.Store.YES,Field.Index.TOKENIZED));
                    for (ItemValue iv : ind.md.getItemValues()) {
                        final ItemPropertyValue[] values = iv.getValues();
                        if(values!=null && values.length>0) {
                            for(ItemPropertyValue val: values){
                                if(val.isSet())
                                    doc.add(new Field(iv.getItemProperty(),val.getLogicalName()+"",Field.Store.YES,Field.Index.UN_TOKENIZED));
                            }
                        }else if (StringUtils.hasLength(iv.getValueText())) {
                            doc.add(new Field(iv.getItemProperty(),iv.getValueText(),Field.Store.YES,Field.Index.UN_TOKENIZED));
                        }

                        //final ItemPropertyValue value = MetadataService.getItemProperty(iv.getItemProperty()).getItemPropertyValue(iv.getValueNumber());
                        //doc.add(new Field(iv.getItemProperty(),value.getLogicalName()+"",Field.Store.YES,Field.Index.UN_TOKENIZED));
                    }
                    writer.addDocument(doc);
                }
                log.debug("gathered indexables. Added ["+col.size()+"] documents to the index");
            } finally {
                writer.optimize();
                writer.close();
            }
            params.setSessionAttribute(PEDAL.indexName,idx);
        } catch (IOException e) {
            e.printStackTrace();
            params.setRequestAttribute("error",e.getMessage());
        }

        return new XMLTransform("<content/>");
    }

    /**
     * Adds indexable objects to the ilst of indexables.
     *
     * @param child the id of item that is mapped to the module.
     * @param languageid the language of the user.
     * @param mpId the marketplace id of the user.
     * @param indexables collection of indexables.
     */
    private void addIndexables(Long child,long languageid,long mpId,final Collection<Indexable> indexables) {
        ContentRevision revision = cpda.getLiveContentRevisionWithEffectiveLanguage(child, languageid, mpId);
        final ContentItem ci = revision.getContentItem();
        if (ci instanceof ContentActivityNode) {
            ContentActivityNode can=(ContentActivityNode) ci;
            final long contentItemId = can.getTargetId();
            if (contentItemId > 0) {
                final ContentItem folder = cpda.getContentItem(contentItemId);
                if (folder instanceof ContentFolder) {
                    ContentFolder f=(ContentFolder) folder;
                    for (Long item : f.getChildren()) {
                        ContentRevision rev=cpda.getLiveContentRevisionWithEffectiveLanguage(item, languageid, mpId);
                        final ContentItem revItem = rev.getContentItem();
                        if (!revItem.getContentType().startsWith(PublishConstants.OBJECTTYPE_CONTENTIMAGE)) {
                            final Metadata md = MetadataService.getMetadataForObject(rev.getObjectId());
                            indexables.add(new Indexable(rev.getTitle(),rev.getObjectType(),revItem.getObjectId(),contentItemId,md));
                        }
                    }
                }
            }
            for (Long ch : can.getChildren()) 
                addIndexables(ch,languageid,mpId,indexables);
        }else
            log.warn("revision.getContentItem():"+ ci.getClass().getName());
    }


    /**
     * Class that holds indexing information.
     */
    private class Indexable{
        /**
         * The title of LearningObject
         */
        private String title;
        /**
         * The type of learning object.
         */
        private String type;
        /**
         * The id of learning object.
         */
        private long id;
        /**
         * The folder containing this learning object.
         */
        private long parentId;
        /**
         * Metadata associated with this learning object.
         */
        private Metadata md;

        /**
         * Constructs a new Indexable object.
         * @param title the title of the learning object
         * @param type the type of the learning object
         * @param id the identifier of the learning object
         * @param parentId the folder containing  learning object
         * @param md the metadata associated with the learning object
         */
        private Indexable(String title, String type, long id, long parentId, Metadata md) {
            this.title = title;
            this.type = type;
            this.id = id;
            this.parentId = parentId;
            this.md = md;
        }
    }
}
