package no.inspera.services.pedal.actions;

import no.inspera.applications.publish.datamodel.ContentActivityNode;
import no.inspera.applications.publish.datamodel.ContentActivityRoot;
import no.inspera.applications.publish.datamodel.ContentRevision;
import no.inspera.applications.publish.structure.StructureElementConverter;
import no.inspera.db.ContentPresentationDataAccess;
import no.inspera.services.log.LogServices;
import no.inspera.services.pedal.LearningObject;
import no.inspera.services.pedal.Module;
import no.inspera.services.pedal.PEDAL;
import no.inspera.services.pedal.engine.AdaptiveEngine;
import no.inspera.services.pedal.utils.StringUtils;
import no.inspera.utils.Parameters;
import org.apache.log4j.Logger;
import org.apache.lucene.analysis.standard.StandardAnalyzer;
import org.apache.lucene.document.Document;
import org.apache.lucene.queryParser.ParseException;
import org.apache.lucene.queryParser.QueryParser;
import org.apache.lucene.search.Hits;
import org.apache.lucene.search.IndexSearcher;
import org.apache.lucene.search.Searcher;
import org.apache.lucene.store.Directory;
import org.apache.lucene.store.RAMDirectory;

import java.io.IOException;
import java.util.Map;

/**
 * The View action displays the module. The sequencing engine is used to ensure the correct sequencing of learning objects.
 *
 * @author <a href="mailto:naimdjon@gmail.com">Naimdjon Takhirov</a>
 * @version $Id: View.java 45074 2008-05-29 10:04:54Z naimdjon $
 * @since 23.12.2007
 */
public class View implements Action {

    /**
     * Logging utility.
     */
    private final transient Logger log = LogServices.getLogger(getClass());

    /**
     * The class that provides data access functionality.
     */
    private final ContentPresentationDataAccess db = ContentPresentationDataAccess.getInstance();

    /**
     * Executes action and displays the module page with the next LearningObject.
     * @param params the current request parameters.
     * @return result of the action. The result is xml transformation that displays a learning object in the module.
     */
    public Result execute(Parameters params) {
        ContentRevision revision = params.getRequestAttribute(PEDAL.course);
        ContentActivityRoot root = (ContentActivityRoot) revision.getContentItem();
        final AdaptiveEngine engine = params.getSessionAttribute(PEDAL.sequencingEngine);
        StringBuffer xml = new StringBuffer("<?xml version=\"1.0\" encoding=\"ISO-8859-1\" ?>");
        xml.append("<content>\n");
        try {
            long moduleId = params.getLong("moduleId");
            if (moduleId > 0) {
                if (engine != null && params.getRequest().getRequestURI().contains("/clear")) {
                    log.debug("refreshing module:"+moduleId);
                    engine.getModule(moduleId).getCurrentState().refresh();
                }
                ContentActivityNode ci = (ContentActivityNode) db.getContentItem(moduleId);
                log.debug("ci.isMandatory():"+ci.isMandatory());
                log.debug("ci.getTargetId() :"+ci.getTargetId() );
                if (ci.isMandatory() && ci.getTargetId() > 0) // pre/post assessments and learning style questionnaire.
                {
                    xml.append("<forwardContent mandatory=\"true\">").append(ci.getTargetId()).append("</forwardContent>");
                    if (engine != null)
                        engine.getModule(moduleId).getCurrentState().markCurrent(new LearningObject(ci.getTargetId(), ci.getContentType()));
                } else if ((ci.getTargetId() > 0 || ci.hasChildren()) && engine!=null) {
                    viewModule(params, engine, xml, moduleId, ci);
                }
            }
            if (engine == null) xml.append("<noAssessments/>");
            if (engine != null) {
                xml.append("<learningstyle>").append(engine.getLearningStyleValue()).append("</learningstyle>");
                xml.append("<level>").append(engine.getLevelValue(moduleId)).append("</level>");
            }
            StructureElementConverter.appendAsActivityPlanXML(root, params.getUserId(), xml);
            //showIndex(params);
        } catch (Exception e) {
            log.error("Exception", e);
        }
        appendModuleLevels(xml,params);
        params.appendPageInfoXML(xml);
        xml.append("</content>\n");
        return new XMLTransform(xml.toString());
    }

    /**
     * Gets the next entry in the chain for the current module.
     *
     * @param params current parameters
     * @param engine the engine
     * @param xml xml buffer
     * @param moduleId current moduleid
     * @param ci activity node
     * @throws IOException if any IO exception occurs.
     * @throws ParseException if any exception occurs when parsing query
     */
    private void viewModule(Parameters params, AdaptiveEngine engine, StringBuffer xml, long moduleId, ContentActivityNode ci) throws IOException, ParseException {
        StringBuffer queryBuf = new StringBuffer("+(");
        queryBuf.append("parentId:").append(ci.getTargetId());
        queryBuf.append(")");
        final StringBuffer query = queryBuf.append(" +").append(engine.getLearningStyleProp()).append(":").append(engine.getLearningStyleValue());
        final String levelValue = engine.getLevelValue(moduleId);
        if(StringUtils.hasLength(levelValue))
            query.append(" +").append(PEDAL.level).append(":").append(levelValue);
        final Module module = engine.getModule(moduleId);
        final Hits hits = search(query.toString(), (RAMDirectory) params.getSessionAttribute(PEDAL.indexName));
        log.debug("hits.length():" + hits.length());
        if (hits.length() > 1) {
            addHits(engine, module, hits);
            LearningObject currentItem = engine.sequence(module);
            if (module.getCurrentState().isAllProcessed()  || currentItem ==null) {
                xml.append("<allProcessed/>");
                if(currentItem!=null)
                    xml.append("<forwardContent>").append(currentItem.getId()).append("</forwardContent>");
            }else {
                log.info("same module:"+currentItem.getId());
                xml.append("<sameModule/>");
                xml.append("<forwardContent>").append(currentItem.getId()).append("</forwardContent>");
            }
        } else if (hits.length() == 1) {
            processSingleHit(engine, xml, ci, module, hits);
        }
    }

    /**
     * Processes search result with single hit.
     *
     * @param engine the engine
     * @param xml xml buffer
     * @param ci activity node
     * @param module current module.
     * @param hits search hits
     * @throws IOException if any IO error occurs.
     */
    private void processSingleHit(AdaptiveEngine engine, StringBuffer xml, ContentActivityNode ci, Module module, Hits hits) throws IOException {
        engine.sequence(module);
        if (module.getCurrentState().isAllProcessed()) xml.append("<allProcessed/>");
        final Document firstHit = hits.doc(0);
        final String id = firstHit.getField("id").stringValue();
        log.debug("id:"+id);
        xml.append("<forwardContent>").append(id).append("</forwardContent>");
        module.getCurrentState().markCurrent(new LearningObject(Long.valueOf(id), ci.getContentType()));
    }


    /**
     * Appends user's level for specific module.
     *
     * @param xml the xml buffer
     * @param params the current request parameters.
     */
    private void appendModuleLevels(StringBuffer xml, Parameters params) {
        Map<Long, PEDAL.Level>  moduleLevels= params.getSessionAttribute( PEDAL.moduleLevels);
        if(moduleLevels==null || moduleLevels.size()<1)return;

        xml.append("<moduleLevels>");
        for (Map.Entry<Long, PEDAL.Level> en : moduleLevels.entrySet())
            xml.append("<module id=\"").append(en.getKey()).append("\" level=\"").append(en.getValue()).append("\"/>");
        xml.append("</moduleLevels>");
    }

    /**
     * Adds hits to the module.
     *
     * @param engine the sequencing engine
     * @param module the current module.
     * @param hits search hits for the module.
     * @throws IOException if any error occurs when accessing documents in  hits.
     */
    private void addHits(AdaptiveEngine engine, Module module, Hits hits) throws IOException {
        for (int i = 0; i < hits.length(); i++) {
            final Document doc = hits.doc(i);
            final LearningObject contentItem = new LearningObject(Long.valueOf(doc.get(PEDAL.Index.id.toString())), doc.get(PEDAL.Index.type.toString()));
            contentItem.setIsExercise(Boolean.valueOf(doc.get("is_exercise")));
            contentItem.setLearningStyle((doc.get(engine.getLearningStyleProp())));
            module.addItem(contentItem);
        }
    }

    /**
     * Performs in-memory search for the learning objects matching query specified by <tt>query</tt>.
     * 
     * @param query the query string.
     * @param dir the directory containing lucene index.
     * @return search hits
     * @throws IOException if any error occurs accessing the index
     * @throws ParseException if any error occurs when parsing the search query.
     */
    private Hits search(String query, Directory dir) throws IOException, ParseException {
        log.debug("query:" + query);
        Searcher searcher = new IndexSearcher(dir);
        return searcher.search(new QueryParser("id", new StandardAnalyzer()).parse(query));
    }
}