package no.inspera.services.pedal.engine;

import no.inspera.services.pedal.LearningObject;
import no.inspera.services.pedal.Module;
import no.inspera.services.pedal.PEDAL;
import no.inspera.services.pedal.utils.StringUtils;
import no.inspera.services.pedal.utils.ICollections;

import java.util.Map;

/**
 * AdaptiveEngine is the engine that adapts LearningObject in the sequencing chain.
 *
 * @author <a href="mailto:naimdjon@gmail.com">Naimdjon Takhirov</a>
 * @version $Id: AdaptiveEngine.java 45074 2008-05-29 10:04:54Z naimdjon $
 * @since 23.12.2007
 */
public class AdaptiveEngine {
    /**
     * A map of modules in the course identified by moduleId key and a Module object.
     */
    private final Map<Long,Module> modules= ICollections.newHashtable();

    /**
     * The sequencing chain.
     */
    protected Chain chain;

    /**
     * The user's learning style.
     */
    protected SequencingMetadataItem learningStyle;

    /**
     * A map of user level for each module. Modules are represented by moduleId key.
     */
    protected Map<Long, PEDAL.Level> levels;

    /**
     * Returns a logical identifier for user's learning style.
     * @return user's learning style.
     */
    public String getLearningStyleValue(){
        return learningStyle.getValue().getLogicalName();
    }

    /**
     * Returns user's level for the specified module.
     *
     * @param moduleId module identifier
     * @return user's level for the specified module.
     */
    public String getLevelValue(long  moduleId){
        final PEDAL.Level level = levels.get(moduleId);
        return StringUtils.checkNull(level);
    }

    /**
     * Returns the learning style metadata element.
     * @return the learning style metadata element.
     */
    public String getLearningStyleProp(){
        return learningStyle.getProperty().getItemPropertyId();
    }

    /**
     * Returns the module identified by specified moduleId.
     * @param moduleId the identifier of the module.
     * @return  Module
     */
    public Module getModule(long moduleId){
        Module module = modules.get(moduleId);
        if (module == null) {
            module= new Module(moduleId);
            modules.put(moduleId,module);
        }
        return module;
    }

    /**
     * Returns string representation of this object
     * @return string representation of this object
     */
    public String toString() {
        return chain.toString();
    }

    /**
     * Causes the sequencing of LearningObjects in the chain.
     *
     * @param module LearningObjects in the module to be sequenced
     * @return next LearningObject
     */
    public LearningObject sequence(Module module) {
        chain.doSequence(module);
        module.getCurrentState().checkAllProcessed(module.getAllItems());
        return module.getCurrentState().getCurrentItem();
    }
}
