package no.inspera.services.pedal.engine;

import no.inspera.services.pedal.Module;
import no.inspera.services.pedal.PEDAL;

import java.util.Map;

/**
 *
 * SequencingBuilder uses builder pattern to build SequencingEngine object.
 * 
 * @author <a href="mailto:naimdjon@gmail.com">Naimdjon Takhirov</a>
 * @version $Id: EngineBuilder.java 45069 2008-05-29 08:25:25Z naimdjon $
 * @since 27.12.2007
 */
public class EngineBuilder {

    /**
     * the sequencing engine to be built.
     */
    private AdaptiveEngine se;

    /**
     * The first entry in the SequencingChain.
     */
    private Chain start;

    /**
     * The last configured entry in SequencingChain.
     */
    private Chain lastConfiguredChain;

    /**
     * Creates  a new SequencingBuilder.
     * @return new SequencingBuilder.
     */
    public static EngineBuilder create(){
        final EngineBuilder builder = new EngineBuilder();
        builder.se= new AdaptiveEngine();
        return builder;
    }

    /**
     * Sets the learning style of a user to the sequencing engine.
     * @param learningStyle the user learning style.
     * @return SequencingBuilder
     */
    public EngineBuilder setLearningStyle(SequencingMetadataItem learningStyle) {
        this.se.learningStyle=learningStyle;
        return this;
    }

    /**
     * Sets the mapping of module to learner level.
     *
     * @param levelMap the level mapping.
     * @return SequencingBuilder
     */
    public EngineBuilder setLevelMap(Map<Long,PEDAL.Level> levelMap) {
        this.se.levels =levelMap;
        return this;
    }

    /**
     * Adds an empty sequence to the chain.
     * @return SequencingBuilder
     */
    public EngineBuilder addEmpty() {
        Sequence empty= new Sequence() {
            public void doSequence(Chain chain, Module module) {
                //System.out.println("empty");
            }
            
            public String toString(){
                return "empty anonymous inner ";
            }
            
        };
        return addSequencingChain(empty);
    }

    /**
     * Adds the specified implementation of <tt>Sequence</tt> to the sequencing chain.
     * @param seq the sequence to be added to the chain.
     * @return SequencingBuilder
     */
    public EngineBuilder addSequencingChain(Sequence seq) {
        Chain chain=new Chain();
        chain.seq=seq;
        if(this.lastConfiguredChain !=null) {
           this.lastConfiguredChain.next = chain;
        } else{
            this.start=chain;
            chain.next= new Chain();
        }
        this.lastConfiguredChain=chain;
        return this;
    }

    /**
     * This method is usually called when we are done with building a sequencing engine.
     * @return SequencingEngine that has been built.
     */
    public AdaptiveEngine build(){
        this.se.chain=start;
        //System.out.println("this.se.chain:"+this.se.chain);
        return se;
    }


}
