package no.inspera.services.pedal.engine;

import no.inspera.services.log.LogServices;
import no.inspera.services.pedal.LearningObject;
import no.inspera.services.pedal.Module;
import no.inspera.services.pedal.PEDAL;
import no.inspera.services.pedal.utils.ICollections;
import org.apache.log4j.Logger;

import java.util.Map;
import java.util.Queue;

/**
 *
 * This class is an implementation of <tt>Sequence</tt> interface. It is reponsible for sequencing LearningObjects based on learning style.
 * <p>
 * At the time of implementation the VAK learning style model is supported.
 * <ul>
 *          <li>visual: the sequencing is done as follows: 1) theory, 2) example, 3) exercise.</li>
 *          <li>auditory: the sequencing is done as follows: 1) theory, 2) example, 3) exercise.</li> 
 *          <li>kinesthetic: the sequencing is done as follows: 1) exercise, 2) example , 3) theory .</li> 
 * </ul>
 *
 * @author <a href="mailto:naimdjon@gmail.com">Naimdjon Takhirov</a>
 * @version $Id: LearningStyleSequencing.java 45069 2008-05-29 08:25:25Z naimdjon $
 * @since 27.12.2007 
 */
public class LearningStyleSequencing implements Sequence {
    private final transient Logger log = LogServices.getLogger(getClass());

    /**
     * A map of learning styles. The key is string identifier for learning style and the LearningObject collector is stored as value 
     */
    private Map<String, LearningStyleContentCollector> contentCollectorMap = ICollections.newMap();

    /**
     * The learning style of the user.
     */
    private SequencingMetadataItem learningStyle;

    /**
     * Constructs a new LearningStyleSequencing object for the specified user learningStyle
     * @param learningStyle user's learning style.
     */
    public LearningStyleSequencing(SequencingMetadataItem learningStyle) {
        this.learningStyle = learningStyle;
        contentCollectorMap.put(PEDAL.LearningStyle.auditory.s(), new AuditoryContentCollector());
        contentCollectorMap.put(PEDAL.LearningStyle.visual.s(), new VisualContentCollector());
        contentCollectorMap.put(PEDAL.LearningStyle.kinesthetic.s(), new KinestheticContentCollector());
    }

    /**
     * The implementation of doSequence method.
     * @param next the next Sequence object in the chain-
     * @param module the current module.
     */
    public void doSequence(Chain next, Module module) {
        Queue<LearningObject> q = new java.util.LinkedList<LearningObject>();
        SequencingState currentState = module.getCurrentState();
        final LearningStyleContentCollector collector = contentCollectorMap.get(learningStyle.getValue().getLogicalName());
        collector.addContentItems(q, module);
        Queue<LearningObject> newQueue = new java.util.LinkedList<LearningObject>();
        log.debug("new queue:"+q);
        for (LearningObject learningObject : q) {
            log.debug(learningObject.getId()+" processed:"+currentState.isProcessed(learningObject)+"....");
            if (!currentState.isProcessed(learningObject)) {
                newQueue.offer(learningObject);
            }
        }
        log.debug("newQueue:"+newQueue);
        final LearningObject peek = newQueue.peek();
        log.debug("peek:"+peek);
        if(peek==null)
            currentState.refresh();
         else
            currentState.markCurrent(peek);
        next.doSequence(module);
    }

    /**
     * Returns string representation of this object.
     * @return string representation of this object.
     */
    public String toString() {
        return "learning style\n";
    }

    /**
     * An interface used to abstract the collection process of items.
     */
    private interface LearningStyleContentCollector {
        /**
         * Add contentitems that are in the module to the queue, that is create a sequence of items based on learning style.
         * @param queue the queue of items. 
         * @param module the module.
         */
        void addContentItems(Queue<LearningObject> queue, Module module);
    }

    /**
     * LearningStyleCollector for users with visual learning style preference.
     */
    private class VisualContentCollector implements LearningStyleContentCollector {

        /**
         * Add contentitems that are in the module to the queue, that is create a sequence of items based on learning style.
         * <p>
         * The sequencing is done as follows:
         * <ol>
         * <li>theory</li>
         * <li>example.</li>
         * <li> exercise.</li>
         * </ol>
         * @param queue the queue of items.
         * @param module the module.
         */
        public void addContentItems(Queue<LearningObject> queue, Module module) {
            if (module.getTheory() != null) {
                queue.add(module.getTheory());
            }
            if (module.getExample() != null) {
                queue.add(module.getExample());
            }
            if (module.getExercise() != null)
                queue.add(module.getExercise());
        }
    }

    /**
     * LearningStyleCollector for users with auditory learning style preference.
     */
    private class AuditoryContentCollector implements LearningStyleContentCollector {
        /**
         * Add contentitems that are in the module to the queue, that is create a sequence of items based on learning style.
         * <p>
         * The sequencing is done as follows:
         * <ol>
         * <li>theory</li>
         * <li>example.</li>
         * <li> exercise.</li>
         * </ol>
         * @param queue the queue of items.
         * @param module the module.
         */
        public void addContentItems(Queue<LearningObject> queue, Module module) {

            if (module.getTheory() != null) {
                queue.add(module.getTheory());
            }
            log.debug("module.getExample():"+module.getExample());
            if (module.getExample() != null) {
                queue.add(module.getExample());
            }
            log.debug("module.getExercise()"+module.getExercise());
            if (module.getExercise() != null)
                queue.add(module.getExercise());
        }
    }

    /**
      * LearningStyleCollector for users with kinesthetic learning style preference.
     */
    private class KinestheticContentCollector implements LearningStyleContentCollector {
        /**
         * Add contentitems that are in the module to the queue, that is create a sequence of items based on learning style.
         * <p>
         * The sequencing is done as follows:
         * <ol>
         * <li> exercise.</li>
         * <li>example.</li>
         * <li>theory</li>
         * </ol>
         * @param queue the queue of items.
         * @param module the module.         
         */
        public void addContentItems(Queue<LearningObject> queue, Module module) {
            if (module.getExercise() != null)
                queue.add(module.getExercise());

            if (module.getExample() != null) {
                queue.add(module.getExample());
            }
            if (module.getTheory() != null) {
                queue.add(module.getTheory());
            }
        }
    }
}
