package no.inspera.services.pedal.engine;

import no.inspera.services.pedal.LearningObject;
import no.inspera.services.pedal.Module;
import no.inspera.services.pedal.utils.ICollections;

import java.util.Collection;
import java.util.concurrent.locks.ReadWriteLock;
import java.util.concurrent.locks.ReentrantReadWriteLock;

/**
 * SequencingState that holds the sequencing information for a module. 
 *
 * @author <a href="mailto:naimdjon@gmail.com">Naimdjon Takhirov</a>
 * @version $Id: SequencingState.java 45069 2008-05-29 08:25:25Z naimdjon $
 * @since 27.12.2007
 */
public class SequencingState {
    //private final transient org.apache.log4j.Logger log = no.inspera.services.log.LogServices.getLogger(getClass());
    /**
     * The currentItem in the queue
     */
    private LearningObject currentItem;

    /**
     * The module this state belongs to
     */
    private Module module;

    /**
     * the boolean flag indicates whether all items are processed
     */
    private boolean allProcessed=false;

    /**
     * Collection of processed items in the module.
     */
    private java.util.Collection<LearningObject> processedItems= ICollections.newSet();

    /**
     * Constructs a new <tt>SequencingState</tt> for the specified module
     * @param module the module of this <tt>SequencingState</tt>
     */
    public SequencingState(Module module) {
        this.module = module;
    }

    /**
     * Returns the current item in the queue under processing.
     * @return the current item in the queue under processing.
     */
    public synchronized LearningObject getCurrentItem() {
          return currentItem;
    }

    /**
     * Marks the specifed item as current in the queue of processing items for the module
     *
     * @param learningObject the item to be marked as current
     */
    public synchronized void markCurrent(LearningObject learningObject) {
        if(learningObject ==null) {
            System.out.printf("null item attempted to be marked for module: %d", module.getModuleId() );
            return;
        }
        ReadWriteLock rwl = new ReentrantReadWriteLock();
        try {
            rwl.writeLock().lock();
            this.currentItem = learningObject;
            this.processedItems.add(currentItem);
        } finally {
            rwl.writeLock().unlock();
        }
        this.checkAllProcessed(this.module.getAllItems());
    }

    /**
     * Checks whether all items in the module are processed.
     *
     * @return true if all items are processed, false otherwise.
     */
    public synchronized boolean isAllProcessed() {
        return this.allProcessed;
    }

    /**
     * Refreshes the queue of processing items by removing current item. Calling this method will result in fresh sequencing state..
     */
    public synchronized void refresh() {
        ReadWriteLock rwl = new ReentrantReadWriteLock();
        try {
            rwl.readLock().lock();
            this.currentItem=null;
            this.processedItems.clear();
            this.allProcessed=false;
        } finally {
            rwl.readLock().unlock();
        }
    }

    /**
     * Checks whether the specified item is processed.
     *
     * @param learningObject item to be checked
     * @return true if the item is processed, false otherwise.
     */
    public synchronized boolean isProcessed(LearningObject learningObject) {
        ReadWriteLock rwl = new ReentrantReadWriteLock();
        try {
            rwl.readLock().lock();
            return allProcessed
                            || this.processedItems.contains(learningObject);
        } finally {
            rwl.readLock().unlock();
        }
    }

    /**
     * Checks whether all items in the specified collection are processed.
     *
     * @param learningObjects collection of items to be checked.
     */
    public synchronized void checkAllProcessed(Collection<LearningObject> learningObjects) {
        ReadWriteLock rwl = new ReentrantReadWriteLock();
        try {
            rwl.readLock().lock();
            this.allProcessed= this.processedItems.containsAll(learningObjects);
        } finally {
            rwl.readLock().unlock();
        }
    }
}
