package no.inspera.services.pedal.services;

import no.inspera.services.pedal.Tag;
import no.inspera.services.pedal.utils.ICollections;
import no.inspera.db.BaseObjectDataAccess;
import no.inspera.db.exception.DataAccessException;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.Collection;


/**
 * Service class for tagging learning objects.  This class contains methods to tag an object and retrieve a set of tags related to a specific object or metadata.
 *
 * @author <a href="mailto:naimdjon@gmail.com">Naimdjon Takhirov</a>
 * @version $Id: TaggingService.java 44417 2008-04-21 18:53:36Z naimdjon $
 * @since 23.02.2008
 */
public class TaggingService {

    /**
     * Tags object with the specified label.
     *
     * @param tag the tag object
     */
    public static void tagObject(Tag tag) {
        BaseObjectDataAccess bo = BaseObjectDataAccess.getInstance();
        final Connection con = bo.getConnection();
        PreparedStatement ps = null;
        try {
            int counter = 1;
            ps = con.prepareStatement("insert into tag(objectid,label,taggingtime,score) values(?,?,?,?)");
            ps.setLong(counter++, tag.getObjectId());
            ps.setString(counter++, tag.getLabel());
            ps.setLong(counter, tag.getTaggingTime().getTimeInMillis());
            final int i = ps.executeUpdate();
            if (i != 1) {
                throw new DataAccessException("Could not insert a new tag to the database");
            }
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            bo.close(ps);
            bo.close(con);
        }
    }

    /**
     * Retrievs tags made by the specified user.
     *
     * @param taggerId the id of tagger.
     * @return an iterable of Tag objects.
     */
    public static Iterable<Tag> getTagsByTaggerId(long taggerId) {
        Collection<Tag> tags = ICollections.newSet();
        BaseObjectDataAccess bo = BaseObjectDataAccess.getInstance();
        final Connection con = bo.getConnection();
        PreparedStatement ps = null;
        try {
            ps = con.prepareStatement("select * from tag where taggerid=?");
            ps.setLong(1, taggerId);
            fillResult(tags, ps);
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            bo.close(ps);
            bo.close(con);
        }
        return tags;
    }

    /**
     * Retrievs objects specified the label for the user.
     * @param label the label of tags
     * @param taggerId the id of tagger
     * @return
     */
    public static Iterable<Long> getObjects(String label, long taggerId) {
        Collection<Long> objectIds = ICollections.newSet();
        BaseObjectDataAccess bo = BaseObjectDataAccess.getInstance();
        final Connection con = bo.getConnection();
        PreparedStatement ps = null;
        try {
            ps = con.prepareStatement("select objectid from tag where label=? and taggerId=?");
            ps.setString(1, label);
            ps.setLong(2, taggerId);
            final ResultSet rs = ps.executeQuery();
            while (rs.next()) {
                objectIds.add(rs.getLong("objectId"));
            }
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            bo.close(ps);
            bo.close(con);
        }
        return objectIds;
    }


    /**
     * Fills the object with the rows from the result set.
     *
     * @param tags a collection where the  retrieved tag is added.
     * @param ps prepared statement
     * @throws SQLException if any data access error occurs.
     */
    private static void fillResult(Collection<Tag> tags, PreparedStatement ps) throws SQLException {
        final ResultSet rs = ps.executeQuery();
        while (rs.next()) {
            Tag tag = new Tag();
            tag.setLabel(rs.getString("label"));
            tag.setObjectId(rs.getLong("objectId"));
            tag.setTaggerId(rs.getLong("taggerid"));
            tags.add(tag);
        }
    }

}
