package no.inspera.services.pedal.utils;

import java.util.*;

/**
 * Utility class to create generic collections. This is usefull and makes the code somewhat cleaner when the generic types in collection
 * include long class names and/or include nested collections. Example:<br/><br/>
 * <code>
 *  Map&lt;Long,Map&lt;String,Map&lt;Long,Long>>> myMap= new HashMap&lt;Long,Map&lt;String,Map&lt;Long,Long>>>();
 * </code>
 *      <br/><br/>
 * can be written: <br/><br/>
 * <code>
 * Map&lt;Long,Map&lt;String,Map&lt;Long,Long>>> myMap= ICollections.newMap();
 * </code>
 * <br/><br/>
 * which would be essentially the same, only with cleaner declaration.
 *
 * @author <a href="mailto:naimdjon@gmail.com">Naimdjon Takhirov</a>
 * @version $Id: ICollections.java 43983 2008-03-25 00:06:35Z naimdjon $
 * @since 28.08.2007
 */
public class ICollections {

    /**
     * Creates a new instance of HashMap
     *
     * @return new map.
     */
     public static <K, V> Map<K, V> newMap() {
        return new HashMap<K, V>();
    }

    /**
     * Constructs an empty <tt>HashMap</tt> with the specified initial
     * capacity and the default load factor (0.75).
     *
     * @return hashmap.
     * @param initialcapacity the initial capacity
     */
     public static <K, V> HashMap<K, V> newHashMap(int initialcapacity) {
        return new HashMap<K, V>(initialcapacity);
    }

    /**
     * Creates a new instance of hashtable.
     * @return hashtable
     */
     public static <K, V> Hashtable<K, V> newHashtable() {
        return new Hashtable<K, V>();
    }

    /**
     * Constructs a new set containing the elements in the specified collection.
     *
     * @param c the collection whose elements are to be placed into this set.
     * @return new set
     */
     public static <E> Set<E> newHashSet(Collection<? extends E> c) {
        return new HashSet<E>(c);
    }

    /**
     * Constructs a new set.
     *
     * @return new set
     */
     public static <E> Set<E> newSet() {
        return new HashSet<E>();
    }

    /**
     * Constructs a new list.
     *
     * @return new list
     */
     public static <E> List<E> newList() {
        return new ArrayList<E>();
    }

    /**
     * Constructs a new list.
     *
     * @return new list
     */
     public static <E> Queue<E> linkedList() {
        return new LinkedList<E>();
    }

    /**
     * Constructs and returns a new  vector.
     *
     * @return new vector
     */
     public static <E> Vector<E> newVect() {
        return new Vector<E>();
    }

    /**
     * Returns a synchronized (thread-safe) map backed by the specified
     * map.  In order to guarantee serial access, it is critical that
     * <strong>all</strong> access to the backing map is accomplished
     * through the returned map.<p>
     *
     * </pre>
     * Failure to follow this advice may result in non-deterministic behavior.
     *
     * <p>The returned map will be serializable if the specified map is
     * serializable.
     *
     * @return a synchronized view of HashMap.
     */
    public static <K, V> Map<K, V> synchronizedMap() {
       return Collections.synchronizedMap(new HashMap<K, V>());
   }

    public static <K, V> SortedMap<K, V> sortedMap() {
       return new TreeMap<K,V>();
   }

}