package no.inspera.services.pedal.utils;

import java.io.PrintWriter;
import java.io.StringWriter;
import java.util.regex.Pattern;
/**
 * Utility class for working with {@link String} class.
 *
 * @author <a href="mailto:naimdjon@gmail.com">Naimdjon Takhirov</a>
 * @version $Id: StringUtils.java 44171 2008-04-02 21:39:14Z naimdjon $
 * @since 20.12.2007
 */
public class StringUtils {
    public static final String ENCODING = System.getProperty("file.encoding");
    private static final transient Pattern numberPattern = Pattern.compile("-?[0-9]+(\\.[0-9]+)?([Ee][\\+-]?[0-9]+)?$");

    /**
     * Compares this string to the specified object.
     * The result is <code>true</code> if and only if both arguments are
     * <code>null</code> or <code>s1</code> is a <code>String</code> object that represents
     * the same sequence of characters as <code>s2</code> object.
     *
     * @param s1 first String
     * @param s2 second String
     * @return <code>true</code> if both arguments are null or {@link String#equals(Object)} returns true
     */
    public static boolean equals(String s1, String s2) {
        return s1 == null ? s2 == null : s1.equals(s2);
    }

    /**
     * Compares <tt>firstString</tt> to the other specified strings. The method permits null <tt>firstString</tt> argument and any of <tt>strings</tt> can be null as well.
     *
     * @param firstString the first string to be compared
     * @param strings an array of strings to be compared with <tt>firstString</tt>
     * @return true if any of <tt>strings</tt> equals to <tt>firstString</tt>.
     */
    public static boolean anyEquals(String firstString, String ... strings) {
        if(strings==null || strings.length<=0)
            return false;

        for (String string : strings)
            if (equals(string, firstString))
                return true;
        return false;
    }

    public static boolean anyEqualsIgnoreCase(String firstString, String ... strings) {
        if(strings==null || strings.length<=0)
            return false;

        for (String string : strings)
            if (equals(string.toLowerCase(), firstString.toLowerCase()))
                return true;
        return false;
    }

    public static boolean anyStartsWithIgnoreCase(String firstString, String ... strings) {
        if(strings==null || strings.length<=0) {
            return false;
        }

        for (String string : strings) if (startsWith(firstString.toLowerCase(), string.toLowerCase())) {
            return true;
        }
        return false;
    }

    /**
     * Prints and returns a string that contains  the stack trace of this exception
     *
     * @param t throwable that needs to be printed
     * @return a string containing stack trace.
     */
    public static String stackTraceToString(Throwable t) {
        StringWriter str = new StringWriter();
        PrintWriter wr = new PrintWriter(str);
        t.printStackTrace(wr);
        return str.toString();
    }


    /**
     * Checks if the string is null. Returns an empty string if this string is null or the string itself otherwise.
     * <p> This method is usefull when we would like to avoid null strings.
     *
     * @param s string to be checked.
     * @return an empty string if this string is null or the string itself otherwise.
     */
    public static String checkNull(String s) {
        return s == null ? "" : s;
    }

    public static String checkNull(Object s) {
        return s == null ? "" : s.toString();
    }


    /**
     * Appends the following value as tag specified by <tt>tagName</tt> if value is not null
     *
     * @param tagName the tag name of xml element
     * @param value   the value to be appended.
     * @param s       stringbuffer to be appended to.
     */
    public static void appendIfNotNull(String tagName, Object value, StringBuffer s) {
        if (value == null) {
            return;
        }
        s.append("<").append(tagName).append(">").append(value).append("</").append(tagName).append(">");
    }

    /**
     * Encodes the given string with the default system encoding.
     *
     * @param str string to be encoded.
     * @return encoded string.
     */
    public static String encode(String str) {
        if (str == null)
            return "";

        try {
            return java.net.URLEncoder.encode(str, ENCODING);
        } catch (java.io.UnsupportedEncodingException e) {
            e.printStackTrace(System.err);
            return "";
        }
    }

    /**
     * Decodes the string with default system encoding.
     *
     * @param str string to be decoded
     * @return decoded string.
     */
    public static String decode(String str) {
        if (str == null)
            return "";

        try {
            return java.net.URLDecoder.decode(str, ENCODING);
        } catch (java.io.UnsupportedEncodingException e) {
            e.printStackTrace(System.err);
            return "";
        }
    }

    /**
     * Checks if the specified string represents a number. Usefull for parsing strings that are numbers.
     *
     * @param str string representing number
     * @return true if the <tt>str</tt> is a number, false otherwise
     */
    public static boolean isNumber(String str) {
        return str != null && str.length() > 0 && numberPattern.matcher(str.trim()).matches();
    }

    /**
     * Checks if the type of the Object is string and checks if it can be parsed as a Number.
     *
     * @param obj Object to be checked.
     * @return true if it can be parsed as a Number.
     */
    public static boolean isParseableNumber(Object obj) {
        return obj != null && (obj instanceof String) && isNumber((String) obj);
    }

    /**
     * Checks if the specified string is true. The check is not case-censitive. The values to be checked true|false are(1|0, true|false, on|off).
     *
     * @param str string to be checked
     * @return true if the str is true, false otherwise.
     */
    public static boolean isTrue(String str) {
        return hasLength(str)
                && (str.equalsIgnoreCase("1")
                            || str.equalsIgnoreCase("on")
                            || str.equalsIgnoreCase("true"));
    }

    /**
     * Checks if the specified string is null or not. NOTE: This method trims the string before checking whether or not it is empty, thus
     * sending an empty string will result in false.
     *
     * @param s string to be checked
     * @return true if the string is empty and false otherwise.
     */
    public static boolean empty(Object s) {
        return s == null || s.toString() == null || s.toString().trim().length() <= 0;
    }

    public static boolean startsWith(String s, String prefix) {
        return s != null && s.startsWith(prefix);
    }


    /**
     * Checks if the specified string is null or not. NOTE: This method trims the string before checking whether or not it is empty, thus
     * sending an empty string will result in false.
     *
     * @param s string to be checked
     * @return true if the string is not empty and false otherwise.
     */
    public static boolean hasLength(String s) {
        return s != null && s.trim().length() > 0;
    }

    /**
     * Tests if this string ends with the specified suffix.
     * @param original
     * @param suffixes the suffixes.
     * @return  <code>true</code> if the character sequence represented by the
     *          <tt>suffixes</tt> argument is a suffix of the character sequence represented by
     *          this object; <code>false</code> otherwise. Note that the
     *          result will be <code>true</code> if the argument is the
     *          empty string or is equal to this <code>String</code> object
     *          as determined by the {@link #equals(Object)} method.
     */
    public static boolean endsWith(String original, String ... suffixes) {
        if(!hasLength(original) || (suffixes==null || suffixes.length<=0)) {
            return false;
        }

        for (String s : suffixes) {
            if(original.endsWith(s)) {
                            return true;
                        }
        }

        return false;
    }

    public static String capitalize(String s) {
        if(!hasLength(s)) {
            return s;
        }

        return s.substring(0, 1).toUpperCase().concat( s.substring(1).toLowerCase());
    }
}
